/*
 * Created on: 11/03/05 15:48:32
 */
package edu.wpi.ejutr.logger;

import edu.wpi.ebob.model.cm.*;
import edu.wpi.ebob.model.*;
import edu.wpi.ebob.*;

import java.util.*;

/**
 * EBOBActionLogger is the class responsible for logging actions run by the 
 * Object Bench.  
 * 
 * This class will maintain a list of all objects that are interacted with using 
 * the bench, and a list of all the methods that were called, what their parameters
 * were, and what order they were called in.
 * 
 * Eventually it will be possible to take an instance of this class and feed it to
 * some other object that will dump the contents to a file.
 * 
 * Implemented as a singleton.
 * 
 * Probably can have it clean out its vector when the plugin gets unloaded.
 * 
 * @author Ben Mohlenhoff - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 */
public class EBOBActionLogger {	
	
	private static EBOBActionLogger me = null;
	
	private ObjectBench theBench;
	// maybe use a different data type, but for now a Vector will work fine
	private Vector actionList;
	
	private  EBOBActionLogger() {
		// attach to Object Bench
		theBench = ObjBenchPlugin.getDefault().getBench();
		actionList = new Vector();
		me = this;
	}		
	
	/**
	 * Returns a handle to the shared EBOBActionLogger instance. (singleton)
	 * @return handle
	 */
	public static EBOBActionLogger getInstance() { 
		if (me == null) {
			me = new EBOBActionLogger();
		}
		return me;
	}
	
	/**
	 * Method responsible for logging method calls.
	 * @param instanceName name of the instance of the calling object
	 * @param methodname   name of the method that was actually called
	 * @param retType	   return type of the method call
	 */
	public void logMethodCall(String instanceName, String methodName, String paramSignature, String retType) {
		actionList.add(new LogMethodCall(instanceName, methodName, paramSignature, retType));
	}
	
	/**
	 * Method responsible for logging the creation of new objects on the bench.
	 * @param objName the fully qualified name of the object.
	 * @param instanceName the instance name of the object.
	 */
	public void logObjectAddition(String objName, String instanceName) {
		actionList.add(new LogObjectAddition(instanceName, objName));
	}
	
	/**
	 * Method responsible for logging the removal of objects from the bench.
	 * @param objName
	 * @param instanceName
	 */
	public void logObjectRemoval(String objName, String instanceName) {
		actionList.add(new LogObjectAddition(instanceName, objName));
	}
	
	/**
	 * Method responsible for logging an assertion.
	 * @param lhs left hand side of the boolean comparison
	 * @param comp the comparison portion of the assertion
	 * @param rhs the right hand side of the boolean comparison
	 * @param res the expected result of the assertion.
	 */
	public void logAssertion(String lhs, String comp, String rhs, boolean res) {
		actionList.add(new LogAssertion(lhs, comp, rhs, res));
	}
	
	/**
	 * Returns an iterator through the list of additions/removals/method calls.
	 * @return an iterator for the action list.
	 */
	public Iterator iterator() {
		return actionList.iterator();
	}
	
	/**
	 * Empties the action list.
	 */
	public void clearLog() {
		actionList.clear();
	}
	
	/**
	 * Returns the last element added to the list.  Does not modify the contents
	 * of the log.
	 * 
	 * @return the last entry made into the log.
	 */
	public EJUTRLogEntry lastEntry() {
		return (EJUTRLogEntry)actionList.lastElement();
	}

}
